@@ -240,7 +240,7 @@ gw = {
       {
         xhr = new XMLHttpRequest();
         xhr.gw_command = command;
-        xhr.open('GET', $root + '/' + encodeURIComponent(gw.form) + '/x?c=' + encodeURIComponent(JSON.stringify(command)), true);
+        xhr.open('GET', $root + '/x?c=' + encodeURIComponent(JSON.stringify(command)), true);
         xhr.onreadystatechange = function() { gw.answer(xhr, after); };
         xhr.send(null);
         return;
@@ -565,7 +565,6 @@ gw = {
         $(last).removeClass('gw-deactivated');
         $(last + '-titlebar').removeClass('gw-deactivated');
       }
-        
     },
     
     raise: function(id, send)
@@ -881,7 +880,7 @@ gw = {
         if (current !== undefined)
         {
           if (current >= 0)
-            $(id + ':' + current).removeClass('gw-table-row-selected');
+            $(id + ':' + current) && $(id + ':' + current).removeClass('gw-table-row-selected');
           tr.addClass('gw-table-row-selected');
           elt.gw_current = row;
         }
@@ -1072,7 +1071,7 @@ gw = {
       xhr.gw_command = 'upload ' + id;
       xhr.gw_id = id;
         
-      xhr.open("POST", $root + '/' + encodeURIComponent(gw.form) + '/u', true);  
+      xhr.open("POST", $root + '/u', true);  
       
       xhr.onreadystatechange = function() 
         {
@@ -1095,11 +1094,6 @@ gw = {
     new AutoComplete({
       selector: $(id),
       cache: false,
-      /*source: function(term, response) 
-        {
-          gw.raise(id, 'completion', [term]);
-          response($(id).gw_completion);
-        }*/
       source: function(term, response) {
         var xhr = $(id).gw_xhr;
         if (xhr)
@@ -1109,7 +1103,7 @@ gw = {
         
         $(id).gw_xhr = xhr = new XMLHttpRequest();
         
-        xhr.open('GET', $root + '/' + encodeURIComponent(gw.form) + '/x?c=' + encodeURIComponent(JSON.stringify(['raise', id, 'completion', [term]])), true);
+        xhr.open('GET', $root + '/x?c=' + encodeURIComponent(JSON.stringify(['raise', id, 'completion', [term]])), true);
         xhr.onreadystatechange = function() {
           if (xhr.readyState == 4)
           {
@@ -1119,6 +1113,9 @@ gw = {
           }
         };
         xhr.send();
+      },
+      onSelect: function(e, term, item) {
+        gw.update(id, 'text', $(id).value);
       }
     });
   },
