@@ -26,6 +26,7 @@
 #include "gdesktop.h"
 #include "gtools.h"
 #include "gpicture.h"
+#include "gapplication.h"
 
 // HTML character entities
 #include "kentities.h"
@@ -44,30 +45,88 @@ Conversion between GDK and long type colors
 #define SCALE(i) ((int)(i * 255.0 / 65535.0 + 0.5))
 #define UNSCALE(d) ((int)(d / 255.0 * 65535.0 + 0.5))
 
+#ifdef GTK3
+static GtkStateFlags _color_style_bg[] = { GTK_STATE_FLAG_NORMAL, GTK_STATE_FLAG_INSENSITIVE, (GtkStateFlags)-1 };
+static GtkStateFlags _color_style_fg[] = { GTK_STATE_FLAG_NORMAL, GTK_STATE_FLAG_ACTIVE, GTK_STATE_FLAG_PRELIGHT, (GtkStateFlags)-1 };
+#else
 static GtkStateType _color_style_bg[] = { GTK_STATE_INSENSITIVE, GTK_STATE_ACTIVE, GTK_STATE_PRELIGHT, GTK_STATE_NORMAL };
 static GtkStateType _color_style_fg[] = { GTK_STATE_ACTIVE, GTK_STATE_PRELIGHT, GTK_STATE_NORMAL };
+#endif
 
-void fill_gdk_color(GdkColor *gcol,gColor color, GdkColormap *cmap)
+#ifdef GTK3
+void fill_gdk_color(GdkColor *gcol, gColor color)
 {
 	int r, g, b;
-	
-	if (!cmap)
-		cmap = gdk_colormap_get_system();
-	
+
 	gt_color_to_rgb(color, &r, &g, &b);
-	
+
 	gcol->red = UNSCALE(r);
 	gcol->green = UNSCALE(g);
 	gcol->blue = UNSCALE(b);
-	
+}
+#else
+void fill_gdk_color(GdkColor *gcol, gColor color, GdkColormap *cmap)
+{
+	int r, g, b;
+
+	if (!cmap)
+		cmap = gdk_colormap_get_system();
+
+	gt_color_to_rgb(color, &r, &g, &b);
+
+	gcol->red = UNSCALE(r);
+	gcol->green = UNSCALE(g);
+	gcol->blue = UNSCALE(b);
+
 	gdk_colormap_alloc_color(cmap, gcol, TRUE, TRUE);
 }
+#endif
 
 gColor get_gdk_color(GdkColor *gcol)
 {
 	return gt_rgb_to_color(SCALE(gcol->red), SCALE(gcol->green), SCALE(gcol->blue));
 }
 
+#ifdef GTK3
+
+static void set_color(GtkWidget *wid, gColor color, void (*func)(GtkWidget *, GtkStateFlags, const GdkRGBA *), bool fg)
+{
+	GdkRGBA gcol;
+	GdkRGBA *pcol;
+	int i;
+	//GtkStyleContext *style;
+	GtkStateFlags st;
+
+	if (color == COLOR_DEFAULT)
+	{
+		pcol = NULL;
+	}
+	else
+	{
+		gt_from_color(color, &gcol);
+		pcol = &gcol;
+
+		/*style = gtk_widget_get_style_context(wid);
+		if (fg)
+			gtk_style_context_get_color(style, GTK_STATE_FLAG_SELECTED, &scol);
+		else
+			gtk_style_context_get_background_color(style, GTK_STATE_FLAG_SELECTED, &scol);*/
+	}
+
+	for (i = 0;; i++)
+	{
+		st = fg ? _color_style_fg[i] : _color_style_bg[i];
+		if (st < 0)
+			break;
+		(*func)(wid, st, pcol);
+	}
+
+	/*if (color != COLOR_DEFAULT)
+		(*func)(wid, GTK_STATE_FLAG_SELECTED, &scol);*/
+}
+
+#else
+
 static void set_color(GtkWidget *wid, gColor color, void (*func)(GtkWidget *, GtkStateType, const GdkColor *), bool fg)
 {
 	GdkColor gcol;
@@ -94,6 +153,9 @@ static void set_color(GtkWidget *wid, gColor color, void (*func)(GtkWidget *, Gt
 	}
 }
 
+#endif
+
+#ifndef GTK3
 gColor get_gdk_fg_color(GtkWidget *wid, bool enabled)
 {
 	GtkStyle* st;
@@ -101,12 +163,18 @@ gColor get_gdk_fg_color(GtkWidget *wid, bool enabled)
 	st=gtk_widget_get_style(wid);
 	return get_gdk_color(&st->fg[enabled ? GTK_STATE_NORMAL : GTK_STATE_INSENSITIVE]);
 }
+#endif
 
 void set_gdk_fg_color(GtkWidget *wid, gColor color)
 {
+#ifdef GTK3
+	set_color(wid, color, gtk_widget_override_color, true);
+#else
 	set_color(wid, color, gtk_widget_modify_fg, true);
+#endif
 }
 
+#ifndef GTK3
 gColor get_gdk_bg_color(GtkWidget *wid, bool enabled)
 {
 	GtkStyle* st;
@@ -114,12 +182,18 @@ gColor get_gdk_bg_color(GtkWidget *wid, bool enabled)
 	st=gtk_widget_get_style(wid);	
 	return get_gdk_color(&st->bg[enabled ? GTK_STATE_NORMAL : GTK_STATE_INSENSITIVE]);
 }
+#endif
 
 void set_gdk_bg_color(GtkWidget *wid,gColor color)
 {
+#ifdef GTK3
+	set_color(wid, color, gtk_widget_override_background_color, false);
+#else
 	set_color(wid, color, gtk_widget_modify_bg, false);
+#endif
 }
 
+#ifndef GTK3
 gColor get_gdk_text_color(GtkWidget *wid, bool enabled)
 {
 	GtkStyle* st;
@@ -127,12 +201,18 @@ gColor get_gdk_text_color(GtkWidget *wid, bool enabled)
 	st=gtk_widget_get_style(wid);	
 	return get_gdk_color(&st->text[enabled ? GTK_STATE_NORMAL : GTK_STATE_INSENSITIVE]);
 }
+#endif
 
 void set_gdk_text_color(GtkWidget *wid,gColor color)
 {
+#ifdef GTK3
+	set_color(wid, color, gtk_widget_override_color, true);
+#else
 	set_color(wid, color, gtk_widget_modify_text, true);
+#endif
 }
 
+#ifndef GTK3
 gColor get_gdk_base_color(GtkWidget *wid, bool enabled)
 {
 	GtkStyle* st;
@@ -140,10 +220,15 @@ gColor get_gdk_base_color(GtkWidget *wid, bool enabled)
 	st=gtk_widget_get_style(wid);
 	return get_gdk_color(&st->base[enabled ? GTK_STATE_NORMAL : GTK_STATE_INSENSITIVE]);
 }
+#endif
 
 void set_gdk_base_color(GtkWidget *wid,gColor color)
 {
+#ifdef GTK3
+	set_color(wid, color, gtk_widget_override_background_color, false);
+#else
 	set_color(wid, color, gtk_widget_modify_base, false);
+#endif
 }
 
 void gt_color_to_rgb(gColor color, int *r, int *g, int *b)
@@ -163,12 +248,54 @@ void gt_color_to_rgba(gColor color, int *r, int *g, int *b, int *a)
 	*b = color & 0xFF;
 	*g = (color >> 8) & 0xFF;
 	*r = (color >> 16) & 0xFF;
-	*a = (color >> 24) & 0xFF;
+	*a = 255 - ((color >> 24) & 0xFF);
 }
 
 gColor gt_rgba_to_color(int r, int g, int b, int a)
 {
-	return (gColor)(b | (g << 8) | (r << 16) | (a << 24));
+	return (gColor)(b | (g << 8) | (r << 16) | ((255 - a) << 24));
+}
+
+void gt_color_to_frgba(gColor color, double *r, double *g, double *b, double *a)
+{
+	*b = (color & 0xFF) / 255.0;
+	*g = ((color >> 8) & 0xFF) / 255.0;
+	*r = ((color >> 16) & 0xFF) / 255.0;
+	*a = 1 - ((color >> 24) & 0xFF) / 255.0;
+}
+
+#ifdef GTK3
+
+void gt_from_color(gColor color, GdkRGBA *rgba)
+{
+	gt_color_to_frgba(color, &rgba->red, &rgba->green, &rgba->blue, &rgba->alpha);
+}
+
+gColor gt_to_color(GdkRGBA *rgba)
+{
+	return gt_frgba_to_color(rgba->red, rgba->green, rgba->blue, rgba->alpha);
+}
+
+void gt_to_css_color(char *css, gColor color)
+{
+	int r, g, b, a;
+	char buffer[8];
+	
+	gt_color_to_rgba(color, &r, &g, &b, &a);
+	if (a == 255)
+		sprintf(css, "#%02X%02X%02X", r, g, b);
+	else
+	{
+		sprintf(buffer, "%.3f", a / 255.0);
+		sprintf(css, "rgba(%d,%d,%d,0.%s)", r, g, b, &buffer[2]);
+	}
+}
+
+#endif
+
+gColor gt_frgba_to_color(double r, double g, double b, double a)
+{
+	return gt_rgba_to_color((int)(r * 255.0), (int)(g * 255.0), (int)(b * 255.0), (int)(a * 255.0));
 }
 
 void gt_rgb_to_hsv(int r, int g, int b, int *H, int *S,int *V )
@@ -337,7 +464,11 @@ gPicture *gt_grab_window(GdkWindow *win, int x, int y, int w, int h)
 	GdkPixbuf *buf = NULL;
 	gPicture *pic;
 	
+#ifdef GTK3
+	gdk_window_get_geometry(win, NULL, NULL, &ww, &hh);
+#else
 	gdk_window_get_geometry(win, 0, 0, &ww, &hh, 0);
+#endif
 	
 	if (w <= 0 || h <= 0)
 	{
@@ -370,7 +501,13 @@ gPicture *gt_grab_window(GdkWindow *win, int x, int y, int w, int h)
 		h = hh - y;
 	
 	if (w > 0 && h > 0)
+	{
+#ifdef GTK3
+		buf = gdk_pixbuf_get_from_window(win, x, y, w, h);
+#else
 		buf = gdk_pixbuf_get_from_drawable(NULL, win, NULL, x, y, 0, 0, w, h);
+#endif
+	}
 	
 	if (w == ow && h == oh)
 		return new gPicture(buf);
@@ -774,7 +911,7 @@ static void add_space(GString *str)
 		g_string_append_c(str, ' ');
 }
 
-char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
+char *gt_html_to_pango_string(const char *html, int len_html, bool newline_are_break)
 {
 	static const char *title[] =
 	{
@@ -798,11 +935,14 @@ char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 	char *token, *markup, **attr;
 	const char *pp;
 	gsize len;
+	bool start_token = false;
 	bool end_token = false;
 	const char **pt;
 	int size = 3; // medium
 	char size_stack[size_stack_len];
 	int size_stack_ptr = 0;
+	bool newline = true;
+	bool inside_par = false;
 	
 	p_end = &html[len_html < 0 ? strlen(html) : len_html];
 	p_markup = NULL;
@@ -818,11 +958,13 @@ char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 			if (p[1] == '/')
 			{
 				p_markup = &p[2];
+				start_token = false;
 				end_token = true;
 			}
 			else
 			{
 				p_markup = &p[1];
+				start_token = true;
 				end_token = false;
 			}
 			continue;
@@ -833,13 +975,24 @@ char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 			if (!p_markup)
 			{
 				g_string_append(pango, "&gt;");
+				newline = false;
 				continue;
 			}
+
 			len = p - p_markup;
+
+			if (p[-1] == '/')
+			{
+				if (len > 0)
+					len--;
+				end_token = true;
+			}
+
 			if (len <= 0)
 			{
+				g_string_append(pango, "&lt;");
 				if (end_token) g_string_append(pango, "/");
-				g_string_append(pango, "&lt;&gt;");
+				g_string_append(pango, "&gt;");
 				p_markup = NULL;
 				continue;
 			}
@@ -852,10 +1005,14 @@ char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 			{
 				if (!strcasecmp(*pt, token))
 				{
-					if (end_token)
-						g_string_append_printf(pango, "</b></span>\n\n");
-					else
+					if (start_token && !end_token)
 						g_string_append_printf(pango, "<span %s><b>", pt[1]);
+					else if (end_token && !start_token)
+					{
+						g_string_append_printf(pango, "</b></span>\n");
+						g_string_append(pango, "<span size=\"smaller\">\n</span>");
+						newline = true;
+					}
 					goto __FOUND_TOKEN;
 				}
 			}
@@ -864,35 +1021,36 @@ char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 			{
 				if (!strcasecmp(*pt, token))
 				{
-					g_string_append_printf(pango, "<%s%s>", end_token ? "/" : "", *pt);
+					if (start_token != end_token)
+						g_string_append_printf(pango, "<%s%s>", end_token ? "/" : "", *pt);
 					goto __FOUND_TOKEN;
 				}
 			}
 			
 			if (!strcasecmp(token, "br") || !strcasecmp(token, "hr"))
 			{
-				if (!end_token)
+				if (start_token)
+				{
 					g_string_append(pango, "\n");
+					newline = true;
+				}
 				goto __FOUND_TOKEN;
 			}
 			
 			if (!strcasecmp(token, "p"))
 			{
-				if (!end_token)
+				if ((end_token || inside_par) && p[1])
 				{
-					if (pango->len > 0)
-					{
-						if (pango->str[pango->len - 1] != '\n')
-							g_string_append(pango, "\n");
-						g_string_append(pango, "\n");
-					}
+					g_string_append(pango, "\n\n");
+					newline = true;
 				}
+				inside_par = start_token;
 				goto __FOUND_TOKEN;
 			}
 			
 			if (!strcasecmp(token, "font"))
 			{
-				if (end_token)
+				if (end_token && !start_token)
 				{
 					if (size_stack_ptr > 0)
 					{
@@ -902,7 +1060,7 @@ char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 					}
 					g_string_append(pango, "</span>");
 				}
-				else
+				else if (start_token && !end_token)
 				{
 					if (size_stack_ptr < size_stack_len)
 						size_stack[size_stack_ptr] = size;
@@ -960,9 +1118,9 @@ char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 			
 			if (!strcasecmp(token, "a") || !strncasecmp(token, "a href", 6))
 			{
-				if (!end_token)
+				if (start_token && !end_token)
 					g_string_append(pango, "<span foreground=\"blue\"><u>");
-				else
+				else if (end_token && !start_token)
 					g_string_append(pango, "</u></span>");
 				goto __FOUND_TOKEN;
 			}
@@ -975,7 +1133,8 @@ char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 				p_markup++;
 			}
 			g_string_append(pango, "&gt;");
-			
+			newline = false;
+
 		__FOUND_TOKEN:
 		
 			g_free(token);
@@ -983,9 +1142,10 @@ char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 			continue;	
 		}
 		
-		if (c == '\n' && !newline)
+		if (c == '\n' && !newline_are_break)
 		{
-			add_space(pango);
+			if (!newline)
+				add_space(pango);
 			continue;
 		}
 		
@@ -994,6 +1154,7 @@ char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 			const char *entity_start = ++p;
 			int entity_len;
 			int entity_unicode;
+			char *endptr;
 			
 // 			if ((p_end - p) >= 6 && !strncasecmp(p, "&nbsp;", 6))
 // 			{
@@ -1018,10 +1179,34 @@ char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 				}
 				else
 				{
-					const struct entity *e = kde_findEntity(entity_start, entity_len);
-					if (e)
+					entity_unicode = -1;
+					endptr = NULL;
+					errno = 0;
+
+					if (*entity_start == '#')
+					{
+						if (entity_start[1] == 'x')
+						{
+							entity_unicode = strtol(&entity_start[2], &endptr, 16);
+							if (entity_unicode <= 0)
+								entity_unicode = -1;
+						}
+						else if (isdigit(entity_start[1]))
+						{
+							entity_unicode = strtol(&entity_start[1], &endptr, 10);
+							if (entity_unicode <= 0)
+								entity_unicode = -1;
+						}
+					}
+					else
+					{
+						const struct entity *e = kde_findEntity(entity_start, entity_len);
+						if (e)
+							entity_unicode = e->code;
+					}
+
+					if (entity_unicode >= 0)
 					{
-						entity_unicode = e->code;
 						g_string_append_unichar(pango, entity_unicode);
 						continue;
 					}
@@ -1030,17 +1215,22 @@ char *gt_html_to_pango_string(const char *html, int len_html, bool newline)
 			
 			g_string_append(pango, "&amp;");
 			p = entity_start - 1;
+			newline = false;
 			continue;
 		}
 	
 		if (c == ' ')
 		{
-			add_space(pango);
+			if (!newline)
+				add_space(pango);
 			continue;
 		}
 			
 		if (!p_markup)
+		{
 			g_string_append(pango, html_entity(*p));
+			newline = false;
+		}
 	}
 	
 	// Sometimes the first markup is not taken into account.
@@ -1148,23 +1338,8 @@ void gt_ensure_visible(GtEnsureVisible *arg, int x, int y, int w, int h)
 	arg->scrollY = -cy;
 }
 
-GtkStyle *gt_get_style(const char *name, int type)
-{
-	GtkSettings *set = gtk_settings_get_default();
-	GtkStyle* st = gtk_rc_get_style_by_paths(set, NULL, name, type);
-	if (!st) st = gtk_widget_get_default_style();
-	return st;
-}
-
-GtkStyle *gt_get_widget_style(const char *name)
-{
-	GtkSettings *set = gtk_settings_get_default();
-	GtkStyle* st = gtk_rc_get_style_by_paths(set, name, NULL, G_TYPE_NONE);
-	if (!st) st = gtk_widget_get_default_style();
-	return st;
-}
-
-
+#ifdef GTK3
+#else
 void gt_pixmap_fill(GdkPixmap *pix, gColor col, GdkGC *gc)
 {
 	GdkColor color;
@@ -1238,6 +1413,7 @@ void gt_pixbuf_render_pixmap_and_mask(GdkPixbuf *pixbuf, GdkPixmap **pixmap_retu
 			*mask_return = NULL;
 	}
 }
+#endif
 
 #if 0
 void gt_pixbuf_replace_color(GdkPixbuf *pixbuf, gColor src, gColor dst, bool noteq)
@@ -1424,7 +1600,6 @@ static void set_layout_from_font(PangoLayout *layout, gFont *font, bool add, int
 	
 	desc = pango_context_get_font_description(font->ct);
 
-	//desc = pango_font_description_copy(pango_layout_get_font_description(layout));
 	if (dpi && dpi != gDesktop::resolution())
 	{
 		int size = pango_font_description_get_size(desc);
@@ -1582,6 +1757,11 @@ gtk_widget_set_can_focus (GtkWidget *widget,
       g_object_notify (G_OBJECT (widget), "can-focus");
     }
 }
+
+void gtk_widget_get_allocation(GtkWidget *widget, GtkAllocation *allocation)
+{
+	*allocation = widget->allocation;
+}
 #endif
 
 static void add_again(GtkWidget *widget, GtkWidget *ignore)
@@ -1595,8 +1775,8 @@ static void add_again(GtkWidget *widget, GtkWidget *ignore)
 		return;
 	
 	parent = GTK_CONTAINER(gtk_widget_get_parent(ignore));
-	control = (gControl *)g_object_get_data(G_OBJECT(widget), "gambas-control");
-	parent_control = (gContainer *)g_object_get_data(G_OBJECT(parent), "gambas-control");
+	control = gt_get_control(widget);
+	parent_control = (gContainer *)gt_get_control(parent);
 	
 	if (control && parent_control)
 	{
@@ -1666,7 +1846,6 @@ void gt_cairo_draw_rect(cairo_t *cr, int x, int y, int w, int h, GB_COLOR color)
 }
 
 // Function partially taken from the GTK+ source code.
-
 cairo_surface_t *gt_cairo_create_surface_from_pixbuf(const GdkPixbuf *pixbuf)
 {
 	gint width = gdk_pixbuf_get_width (pixbuf);
@@ -1681,7 +1860,7 @@ cairo_surface_t *gt_cairo_create_surface_from_pixbuf(const GdkPixbuf *pixbuf)
 	static const cairo_user_data_key_t key = { 0 };
 	int j;
 
-	//fprintf(stderr, "gt_cairo_create_surface_from_pixbuf: %d %d\n", width, height);
+	//fprintf(stderr, "gt_cairo_create_surface_from_pixbuf: %d %d / %d\n", width, height, n_channels);
 	
 	if (n_channels == 3)
 		format = CAIRO_FORMAT_RGB24;
@@ -1756,7 +1935,6 @@ cairo_surface_t *gt_cairo_create_surface_from_pixbuf(const GdkPixbuf *pixbuf)
 	return surface;
 }
 
-
 void gt_cairo_draw_pixbuf(cairo_t *cr, GdkPixbuf *pixbuf, float x, float y, float w, float h, float opacity, GB_RECT *source)
 {
 	cairo_surface_t *surface;
@@ -1820,3 +1998,331 @@ void gt_cairo_draw_pixbuf(cairo_t *cr, GdkPixbuf *pixbuf, float x, float y, floa
 	if (source)
 		g_object_unref(pixbuf);
 }
+
+
+// Style management
+
+#define NUM_STYLES 12
+
+#ifdef GTK3
+static int _style_context_loaded = 0;
+static GtkStyleContext *_style_context[NUM_STYLES];
+#else
+static int _style_loaded = 0;
+static GtkStyle *_style[NUM_STYLES];
+#endif
+
+static int type_to_index(GType type)
+{
+	if (type == GTK_TYPE_ENTRY)
+		return 1;
+	else if (type == GTK_TYPE_LAYOUT)
+		return 2;
+	else if (type == GTK_TYPE_TOOLTIP)
+		return 3;
+	else if (type == GTK_TYPE_SCROLLBAR)
+		return 4;
+	else if (type == GTK_TYPE_SCROLLED_WINDOW)
+		return 5;
+	else if (type == GTK_TYPE_CHECK_BUTTON)
+		return 6;
+	else if (type == GTK_TYPE_RADIO_BUTTON)
+		return 7;
+	else if (type == GTK_TYPE_FRAME)
+		return 8;
+	else if (type == GTK_TYPE_LABEL)
+		return 9;
+	else if (type == GTK_TYPE_BUTTON)
+		return 10;
+	else if (type == GTK_TYPE_WINDOW)
+		return 11;
+	else
+		return 0;
+}
+
+#ifdef GTK3
+
+const char *gt_get_style_class(GType type)
+{
+	static const char *_class[] = {
+		GTK_STYLE_CLASS_DEFAULT, GTK_STYLE_CLASS_ENTRY, GTK_STYLE_CLASS_BACKGROUND, GTK_STYLE_CLASS_TOOLTIP,
+		GTK_STYLE_CLASS_SCROLLBAR, GTK_STYLE_CLASS_DEFAULT, GTK_STYLE_CLASS_CHECK, GTK_STYLE_CLASS_RADIO,
+		GTK_STYLE_CLASS_FRAME, GTK_STYLE_CLASS_BACKGROUND, GTK_STYLE_CLASS_BUTTON, GTK_STYLE_CLASS_DEFAULT
+	};
+
+	int index = type_to_index(type);
+	if (index < 0)
+		return NULL;
+	else
+		return _class[index];
+}
+
+GtkStyleContext *gt_get_style(GType type)
+{
+	int index = type_to_index(type);
+	if (index < 0)
+		return NULL;
+
+	if ((_style_context_loaded & (1 << index)) == 0)
+	{
+		GtkStyleContext *style = gtk_style_context_new();
+		GtkWidgetPath *path = gtk_widget_path_new();
+		const char *klass = gt_get_style_class(type);
+
+		if (klass)
+			gtk_style_context_add_class(style, klass);
+
+		gtk_widget_path_append_type(path, type);
+		gtk_style_context_set_path(style, path);
+		//gtk_widget_path_unref(path);
+
+		_style_context[index] = style;
+		_style_context_loaded |= (1 << index);
+	}
+
+	return _style_context[index];
+}
+
+#else
+
+static GtkStyle *get_style(const char *name, GType type)
+{
+	GtkSettings *set = gtk_settings_get_default();
+	GtkStyle* st = gtk_rc_get_style_by_paths(set, NULL, name, type);
+	if (!st) st = gtk_widget_get_default_style();
+	return st;
+}
+
+static GtkStyle *get_widget_style(const char *name)
+{
+	GtkSettings *set = gtk_settings_get_default();
+	GtkStyle* st = gtk_rc_get_style_by_paths(set, name, NULL, G_TYPE_NONE);
+	if (!st) st = gtk_widget_get_default_style();
+	return st;
+}
+
+GtkStyle *gt_get_style(GType type)
+{
+	int index = type_to_index(type);
+	if (index < 0)
+		return NULL;
+
+	if ((_style_loaded & (1 << index)) == 0)
+	{
+		if (type == GTK_TYPE_TOOLTIP)
+			_style[index] = get_widget_style("gtk-tooltip");
+		else
+			_style[index] = get_style(g_type_name(type), type);
+
+		_style_loaded |= (1 << index);
+	}
+
+	return _style[index];
+}
+
+#endif
+
+void gt_get_style_property(GType type, const char *name, void *pvalue)
+{
+#ifdef GTK3
+	gtk_style_context_get_style(gt_get_style(type), name, pvalue, NULL);
+#else
+	gtk_style_get(gt_get_style(type), type, name, pvalue, (char *)NULL);
+#endif
+}
+
+
+#ifdef GTK3
+
+// Draw a styled border
+
+void gt_draw_border(cairo_t *cr, GtkStyleContext *st, GtkStateFlags state, int border, gColor color, int x, int y, int w, int h, bool bg)
+{
+	if (border == BORDER_NONE)
+		return;
+
+	if (w < 2 || h < 2)
+		return;
+
+	if (border == BORDER_PLAIN)
+	{
+		gt_cairo_draw_rect(cr, x, y, w, h, color);
+		return;
+	}
+
+	if (strcmp(gApplication::getStyleName(), "oxygen-gtk") == 0)
+	{
+		x -= 3;
+		w += 6;
+	}
+
+	gboolean interior_focus;
+
+	if (border == BORDER_RAISED)
+	{
+		st = gt_get_style(GTK_TYPE_BUTTON);
+		gtk_style_context_set_state(st, state);
+		//gtk_style_context_add_class(st, GTK_STYLE_CLASS_BUTTON);
+		gtk_render_frame(st, cr, x, y, w, h);
+	}
+	else if (border == BORDER_SUNKEN)
+	{
+		st = gt_get_style(GTK_TYPE_ENTRY);
+		gtk_style_context_set_state(st, state);
+		//gtk_style_context_add_class(st, GTK_STYLE_CLASS_ENTRY);
+		if (bg)
+			gtk_render_background(st, cr, x, y, w, h);
+		gtk_render_frame(st, cr, x, y, w, h);
+	}
+	else if (border == BORDER_ETCHED)
+	{
+		st = gt_get_style(GTK_TYPE_FRAME);
+		gtk_style_context_set_state(st, state);
+		//gtk_style_context_add_class(st, GTK_STYLE_CLASS_FRAME);
+		gtk_render_frame(st, cr, x, y, w, h);
+	}
+
+	if (state & GTK_STATE_FLAG_FOCUSED)
+	{
+		gtk_style_context_get_style (st, "interior-focus", &interior_focus, NULL);
+		if (!interior_focus)
+			gtk_render_focus(st, cr, x, y, w, h);
+	}
+}
+#endif
+
+#ifdef GTK3
+
+void gt_widget_set_color(GtkWidget *widget, bool fg, gColor color, const char *name, const GdkRGBA *def_color)
+{
+	if (color == COLOR_DEFAULT)
+	{
+		if (name)
+			gtk_widget_override_symbolic_color(widget, name, def_color);
+		
+		if (fg)
+			gtk_widget_override_color(widget, GTK_STATE_FLAG_NORMAL, NULL);
+		else
+			gtk_widget_override_background_color(widget, GTK_STATE_FLAG_NORMAL, NULL);
+	}
+	else
+	{
+		GdkRGBA rgba;
+		gt_from_color(color, &rgba);
+		
+		if (name)
+			gtk_widget_override_symbolic_color(widget, name, &rgba);
+		
+		if (fg)
+			gtk_widget_override_color(widget, GTK_STATE_FLAG_NORMAL, &rgba);
+		else
+			gtk_widget_override_background_color(widget, GTK_STATE_FLAG_NORMAL, &rgba);
+	}
+}
+
+bool gt_style_lookup_color(GtkStyleContext *style, const char **names, const char **pname, GdkRGBA *rgba)
+{
+	const char **p = names;
+	const char *name;
+
+	while((name = *p++))
+	{
+		if (gtk_style_context_lookup_color(style, name, rgba))
+			break;
+	}
+
+	if (name)
+	{
+		if (pname)
+			*pname = name;
+		return FALSE;
+	}
+	else
+		return TRUE;
+}
+
+#endif
+
+bool gt_grab(GtkWidget *widget, bool owner_event, guint32 time)
+{
+	GdkWindow *win = gtk_widget_get_window(widget);
+	int ret;
+
+#ifdef GTK3
+
+	GdkDevice *pointer = gdk_device_manager_get_client_pointer(gdk_display_get_device_manager(gdk_display_get_default()));
+	GdkDevice *keyboard = gdk_device_get_associated_device(pointer);
+
+	ret = gdk_device_grab(pointer, win, GDK_OWNERSHIP_APPLICATION, owner_event,
+		(GdkEventMask)(GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK),
+		gdk_window_get_cursor(win),
+		time);
+
+	if (ret == GDK_GRAB_SUCCESS)
+	{
+		ret = gdk_device_grab(keyboard, win, GDK_OWNERSHIP_APPLICATION, owner_event,
+			(GdkEventMask)(GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK),
+			NULL, time);
+
+		if (ret == GDK_GRAB_SUCCESS)
+			return FALSE;
+
+		gdk_device_ungrab(pointer, GDK_CURRENT_TIME);
+	}
+
+#else
+
+	ret = gdk_pointer_grab(win, owner_event,
+		(GdkEventMask)(GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK),
+		NULL,
+#if GTK_CHECK_VERSION(2, 18, 0)
+		gdk_window_get_cursor(win),
+#else
+		NULL,
+#endif
+		time);
+
+	if (ret == GDK_GRAB_SUCCESS)
+	{
+		ret = gdk_keyboard_grab(win, owner_event, time);
+
+		if (ret == GDK_GRAB_SUCCESS)
+			return FALSE;
+
+		gdk_pointer_ungrab(GDK_CURRENT_TIME);
+	}
+
+#endif
+
+	fprintf(stderr, "gb.gtk: warning: grab failed: %d\n", ret);
+	return TRUE;
+}
+
+void gt_ungrab(void)
+{
+#ifdef GTK3
+	GdkDevice *pointer = gdk_device_manager_get_client_pointer(gdk_display_get_device_manager(gdk_display_get_default()));
+	GdkDevice *keyboard = gdk_device_get_associated_device(pointer);
+
+	gdk_device_ungrab(pointer, GDK_CURRENT_TIME);
+	gdk_device_ungrab(keyboard, GDK_CURRENT_TIME);
+#else
+	gdk_pointer_ungrab(GDK_CURRENT_TIME);
+	gdk_keyboard_ungrab(GDK_CURRENT_TIME);
+#endif
+}
+
+#ifdef GTK3
+void gt_widget_reparent(GtkWidget *widget, GtkWidget *new_parent)
+{
+  GtkWidget *parent = gtk_widget_get_parent(widget);
+  
+  if (parent == new_parent)
+    return;
+  
+  g_object_ref(widget);
+  gtk_container_remove(GTK_CONTAINER(parent), widget);
+  gtk_container_add(GTK_CONTAINER(new_parent), widget);
+  g_object_unref(widget);
+}
+#endif
