@@ -676,7 +676,7 @@ void MyFrame::paintEvent(QPaintEvent *e)
 {
 	QPainter painter(this);
 
-	if (_bg)
+	if (_bg || (_pixmap && _pixmap->hasAlphaChannel()))
 	{
 		CWIDGET *window = CWidget::get(parentWidget());
 		GB_COLOR col = CWIDGET_get_background(window);
@@ -1342,7 +1342,7 @@ BEGIN_METHOD(Container_FindChild, GB_INTEGER x; GB_INTEGER y)
 		if (w->geometry().contains(VARG(x), VARG(y)))
 		{
 			control = CWidget::get(w);
-			if (control)
+			if (control && control != THIS)
 			{
 				GB.ReturnObject(control);
 				return;
@@ -1377,7 +1377,7 @@ BEGIN_METHOD(Container_unknown, GB_VALUE x; GB_VALUE y)
 		return;
 	}
 	
-	GB.Deprecated("gb.qt4", "Container.Find", "Container.FindChild");
+	GB.Deprecated(QT_NAME, "Container.Find", "Container.FindChild");
 	
 	if (GB.Conv(ARG(x), GB_T_INTEGER) || GB.Conv(ARG(y), GB_T_INTEGER))
 		return;
@@ -1425,7 +1425,7 @@ GB_DESC CContainerDesc[] =
 
 	GB_EVENT("BeforeArrange", NULL, NULL, &EVENT_BeforeArrange),
 	GB_EVENT("Arrange", NULL, NULL, &EVENT_Arrange),
-	GB_EVENT("NewChild", NULL, "(Control)Control", &EVENT_Insert),
+	GB_EVENT("NewChild", NULL, "(Child)Control", &EVENT_Insert),
 
 	GB_END_DECLARE
 };
