@@ -33,7 +33,7 @@
 #include "serializer.h"
 #include "parser.h"
 
-#include "../gambas.h"
+#include "gambas.h"
 #include "html/gb.xml.html.h"
 
 GB_INTERFACE GB EXPORT;
@@ -104,7 +104,7 @@ extern "C"
 
         (void *)ThrowXMLParseException,
 
-    #if defined(OS_MACOSX) || defined(__APPLE__)
+    #if defined(OS_MACOSX) || defined(__APPLE__) || defined(__CYGWIN__)
         (void*)memrchr,
     #endif
 
