@@ -45,11 +45,16 @@
 
 #include "gb_arch_temp.h"
 
-/* main archive project (used only if gbx is run with -x flag) */
+//#define DEBUG_COMP 1
+
+// main archive project (used only if gbx is run with -x flag)
 ARCHIVE *ARCHIVE_main = NULL;
 
-static ARCHIVE *_archive_list = NULL;
+// Additional library search directory used in debug mode, normally '~/.config/share/gambas3/lib'
+char *ARCHIVE_path = NULL;
 
+static char *_local_path = NULL;
+static ARCHIVE *_archive_list = NULL;
 static char *arch_pattern = NULL;
 static int arch_index = 0;
 static ARCH *arch_dir = NULL;
@@ -75,6 +80,11 @@ ARCHIVE *ARCHIVE_create(const char *name, const char *path)
 	return arch;
 }
 
+static void error_ARCHIVE_load_exported_class(COMPONENT *current)
+{
+	COMPONENT_current = current;
+}
+
 void ARCHIVE_load_exported_class(ARCHIVE *arch, int pass)
 {
 	char *buffer;
@@ -90,163 +100,193 @@ void ARCHIVE_load_exported_class(ARCHIVE *arch, int pass)
 		return;
 
 	current = COMPONENT_current;
-
 	COMPONENT_current = (COMPONENT *)arch->current_component;
 
-	/* COMPONENT_current is set => it will look in the archive */
-
-	#if DEBUG_COMP
-		fprintf(stderr, "load_exported_class: %s (component: %s)\n", arch->name, COMPONENT_current ? COMPONENT_current->name : "?");
-	#endif
-
-	if (!FILE_exist(".list"))
-		return;
-
-	if (pass & AR_FIND_ONLY)
+	ON_ERROR_1(error_ARCHIVE_load_exported_class, current)
 	{
-		#if DEBUG_COMP
-			fprintf(stderr, "<Find pass>\n");
-		#endif
-
-		STREAM_load(".list", &buffer, &len);
-		/* The file must end with a newline !*/
-		buffer[len - 1] = 0;
+		/* COMPONENT_current is set => it will look in the archive */
 
 		#if DEBUG_COMP
-			fprintf(stderr, "-----------\n%s\n----------\n\n", buffer);
+			fprintf(stderr, "load_exported_class: %s (component: %s)\n", arch->name, COMPONENT_current ? COMPONENT_current->name : "?");
 		#endif
 
-		ARRAY_create(&arch->exported);
-
-		name = strtok(buffer, "\n");
-		while (name)
+		if (FILE_exist(".list"))
 		{
-			#if DEBUG_COMP
-				fprintf(stderr, "Check %s global\n", name);
-			#endif
-
-			len = strlen(name);
-			optional = FALSE;
-
-			for(;;)
+			if (pass & AR_FIND_ONLY)
 			{
-				c = name[len - 1];
-				if (c == '?')
+				#if DEBUG_COMP
+					fprintf(stderr, "<Find pass>\n");
+				#endif
+
+				STREAM_load(".list", &buffer, &len);
+				/* The file must end with a newline !*/
+				buffer[len - 1] = 0;
+
+				#if DEBUG_COMP
+					fprintf(stderr, "-----------\n%s\n----------\n\n", buffer);
+				#endif
+
+				ARRAY_create(&arch->exported);
+
+				name = strtok(buffer, "\n");
+				while (name)
 				{
-					optional = TRUE;
-					len--;
+					#if DEBUG_COMP
+						fprintf(stderr, "Check %s global\n", name);
+					#endif
+
+					len = strlen(name);
+					optional = FALSE;
+
+					for(;;)
+					{
+						c = name[len - 1];
+						if (c == '?')
+						{
+							optional = TRUE;
+							len--;
+						}
+						else if (c == '!')
+							len--;
+						else
+							break;
+					}
+
+					name[len] = 0;
+
+					/*
+					class = CLASS_look_global(name, strlen(name));
+
+					if (class)
+					{
+						#if DEBUG_COMP
+							fprintf(stderr, "...override!\n");
+						#endif
+						CLASS_load(class);
+						CLASS_check_global(class);
+					}
+					else
+						class = CLASS_find_global(name);*/
+
+					if (!optional || CLASS_look_global(name, len) == NULL)
+					{
+						class = CLASS_find_global(name);
+						CLASS_check_global(class);
+
+						#if DEBUG_COMP
+							fprintf(stderr, "Add to load: %p %s\n", class, name);
+						#endif
+						class->component = COMPONENT_current;
+						*((CLASS **)ARRAY_add(&arch->exported)) = class;
+					}
+
+					name = strtok(NULL, "\n");
 				}
-				else if (c == '!')
-					len--;
-				else
-					break;
+
+				FREE(&buffer);
 			}
 
-			name[len] = 0;
-
-			/*
-			class = CLASS_look_global(name, strlen(name));
-
-			if (class)
+			if (pass & AR_FIND_ONLY) // That way the 'pass' flag is always ignored.
 			{
 				#if DEBUG_COMP
-					fprintf(stderr, "...override!\n");
+					fprintf(stderr, "<Load pass>\n");
 				#endif
-				CLASS_load(class);
-				CLASS_check_global(class);
+
+				for (i = 0; i < ARRAY_count(arch->exported); i++)
+				{
+					#if DEBUG_COMP
+						fprintf(stderr, "load %p %s\n", arch->exported[i], arch->exported[i]->name);
+					#endif
+					CLASS_load(arch->exported[i]);
+				}
+
+				ARRAY_delete(&arch->exported);
+				arch->exported_classes_loaded = TRUE;
 			}
-			else
-				class = CLASS_find_global(name);*/
-
-			class = CLASS_find_global(name);
-			CLASS_check_global(class);
-
-			if (optional)
-			{
-				if (CLASS_look_global(name, len))
-					class = NULL;
-			}
-
-			if (class)
-			{
-				#if DEBUG_COMP
-					fprintf(stderr, "Add to load: %p %s\n", class, name);
-				#endif
-				class->component = COMPONENT_current;
-				*((CLASS **)ARRAY_add(&arch->exported)) = class;
-			}
-
-			name = strtok(NULL, "\n");
 		}
-
-		FREE(&buffer);
-	}
-
-	if (pass & AR_FIND_ONLY)
-	{
-		#if DEBUG_COMP
-			fprintf(stderr, "<Load pass>\n");
-		#endif
-
-		for (i = 0; i < ARRAY_count(arch->exported); i++)
-		{
-			#if DEBUG_COMP
-				fprintf(stderr, "load %p %s\n", arch->exported[i], arch->exported[i]->name);
-			#endif
-			CLASS_load(arch->exported[i]);
-		}
-
-		ARRAY_delete(&arch->exported);
-		arch->exported_classes_loaded = TRUE;
 	}
+	END_ERROR
 
 	COMPONENT_current = current;
 }
 
-#if 0
-static void load_component(char *name)
-{
-	COMPONENT *comp;
-
-	comp = COMPONENT_create(name);
-	COMPONENT_load(comp);
-}
-#endif
-
-/*static void load_dependencies(ARCHIVE *arch)
-{
-	char *buffer;
-	int len;
-
-	STREAM_load(".startup", &buffer, &len);
-	PROJECT_analyze_startup(buffer, len, load_component);
-	FREE(&buffer, "load_dependencies");
-}*/
+// Archive path is an absolute path or :<vendor>/<library>:<version>
 
 static char *exist_library(const char *dir, const char *name)
 {
 	char *path;
+	char *n;
+	char *p;
 
-	path = (char *)FILE_cat(dir, name, NULL);
+	n = STRING_new_zero(name);
+
+	path = (char *)FILE_cat(dir, n, NULL);
 	if (FILE_exist(path))
-		return path;
-	else
-		return NULL;
+		goto __RETURN_PATH;
+
+	n = STRING_add(n, ".gambas", -1);
+	path = (char *)FILE_cat(dir, n, NULL);
+	if (FILE_exist(path))
+		goto __RETURN_PATH;
+
+	p = rindex(n, ':');
+	if (p)
+	{
+		strcpy(p, ".gambas");
+		path = (char *)FILE_cat(dir, n, NULL);
+		if (FILE_exist(path))
+			goto __RETURN_PATH;
+	}
+
+	path = NULL;
+
+__RETURN_PATH:
+
+	STRING_free(&n);
+	return path;
 }
 
 void ARCHIVE_load(ARCHIVE *arch, bool load_exp)
 {
 	char *path;
+	char *dir;
+	const char *name;
+	char *env;
 
 	if (arch->path)
 	{
-		if (EXEC_debug)
-			path = (char *)arch->path;
-		else
+		if (*arch->path == ':')
 		{
-			path = exist_library(PROJECT_path, arch->name);
-			if (!path) path = exist_library("/usr/bin", arch->name);
-			if (!path) path = exist_library("/bin", arch->name);
+			if (!_local_path)
+			{
+				env = getenv("XDG_DATA_HOME");
+				if (env && *env)
+					_local_path = STRING_new_zero(FILE_cat(env, "gambas3/lib", NULL));
+				else
+					_local_path = STRING_new_zero(FILE_cat(FILE_get_home(), ".local/share/gambas3/lib", NULL));
+			}
+
+			name = &arch->path[1];
+		}
+		else
+			name = FILE_get_name(arch->path);
+
+		// For backward-compatibility, library name is searched in the current folder too, without the vendor.
+
+		if (!(path = exist_library(PROJECT_path, FILE_get_name(name))))
+		{
+			if (!_local_path ||  !(path = exist_library(_local_path, name)))
+			{
+				if (!ARCHIVE_path || !(path = exist_library(ARCHIVE_path, name)))
+				{
+					if (!(path = exist_library(COMPONENT_path, name)))
+					{
+						dir = STRING_new_zero(FILE_cat(PROJECT_exec_path, "bin", NULL));
+						path = exist_library(dir, name);
+						STRING_free(&dir);
+					}
+				}
+			}
 		}
 
 		if (!path || !FILE_exist(path))
@@ -299,8 +339,6 @@ void ARCHIVE_delete(ARCHIVE *arch)
 
 void ARCHIVE_init(void)
 {
-	//ARCH_create(path);
-	//ARCHIVE_main = _arch_list;
 }
 
 
@@ -311,6 +349,8 @@ void ARCHIVE_exit(void)
 
 	STRING_free(&arch_pattern);
 	STRING_free(&arch_prefix);
+	STRING_free(&ARCHIVE_path);
+	STRING_free(&_local_path);
 }
 
 
@@ -320,19 +360,19 @@ bool ARCHIVE_find_from_path(ARCHIVE **parch, const char **ppath)
 {
 	int i;
 	CLASS *class;
-	
+
 	if (*parch)
 		return FALSE;
-	
+
 	if (COMPONENT_current && COMPONENT_current->archive)
 		*parch = COMPONENT_current->archive;
 	else if (CP && CP->component && CP->component->archive)
 		*parch = CP->component->archive;
 	else
 		*parch = NULL;
-	
+
 	//fprintf(stderr, "ARCHIVE_find_from_path: %s (%s)\n", *ppath, *parch ? (*parch)->name : "NULL");
-	
+
 	if (strncmp(*ppath, ".../", 4) == 0)
 	{
 		*ppath += 4;
@@ -346,7 +386,7 @@ bool ARCHIVE_find_from_path(ARCHIVE **parch, const char **ppath)
 			*ppath += 3;
 			if (*parch == NULL || *parch == ARCHIVE_main)
 				continue;
-			
+
 			while (i < STACK_frame_count)
 			{
 				class = STACK_frame[i].cp;
@@ -364,20 +404,20 @@ bool ARCHIVE_find_from_path(ARCHIVE **parch, const char **ppath)
 						break;
 					}
 				}
-				
+
 				i++;
 			}
-			
+
 			if (i == STACK_frame_count)
 				*parch = NULL;
 		}
 	}
-	
+
 	if (*parch == NULL && EXEC_arch)
 		*parch = ARCHIVE_main;
-	
+
 	//fprintf(stderr, "--> '%s' / %s\n", *parch ? (*parch)->name : "(null)", *ppath);
-	
+
 	return *parch == NULL;
 }
 
@@ -457,7 +497,7 @@ bool ARCHIVE_is_dir(ARCHIVE *arch, const char *path)
 
 void ARCHIVE_stat(ARCHIVE *arch, const char *path, FILE_STAT *info)
 {
-	ARCHIVE_FIND find;
+	ARCHIVE_FIND find = { 0 };
 	struct stat buf;
 
 	//if (get_current(&arch))
